import PropTypes from 'prop-types';

import HoverDisplay from './HoverDisplay';
import DrawerDisplay from './DrawerDisplay';
import TooltipContent from './TooltipContent';

/**
 * A tooltip display container. This is what you normally think of as the
 * tooltip itself. It will render either as a hovering card or as a drawer at
 * the bottom of the screen, depending on screen size. The content of the
 * tooltip is specified by the tooltip object passed in as the `tooltip` prop.
 * If you want to use one tooltip display element for multiple tooltip targets
 * you should render one `TooltipDisplay` and many `TooltipTriggers`, passing
 * them all the same `tooltip` object returned from one call of the
 * `useTooltip` hook. That would be a good approach if, say, you have many path
 * nodes in an SVG and you want each path to trigger a the same tooltip with
 * unique content.
 */
const TooltipDisplay = function TooltipDisplay(props) {
  const {
    tooltip,
    ...forwardProps
  } = props;

  const {
    content,
    props: childProps,
    isDesktop,
  } = tooltip.config;

  const WrapperComponent = isDesktop ? HoverDisplay : DrawerDisplay;

  return (
    <WrapperComponent tooltip={tooltip} {...forwardProps}>
      <TooltipContent
        content={content}
        props={childProps}
      />
    </WrapperComponent>
  );
};

TooltipDisplay.defaultProps = {};

TooltipDisplay.propTypes = {
  /**
   * A tooltip object as provided by the `useTooltip` hook
   */
  tooltip: PropTypes.shape({
    config: PropTypes.shape({
      content: PropTypes.oneOfType([
        PropTypes.elementType,
        PropTypes.node,
      ]),
      props: PropTypes.shape({}),
      isDesktop: PropTypes.bool,
    }),
  }).isRequired,
};

export default TooltipDisplay;
